package org.monaka.book.android.camera;

import java.io.IOException;

import android.content.Context;
import android.graphics.ImageFormat;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.media.AudioManager;
import android.media.SoundPool;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;

public class CameraPreview extends SurfaceView implements Callback {
    private SurfaceHolder holder;
    protected Camera camera;
    private PictureCallback pictureCallback;
    private PreviewCallback previewCallback;
    private SoundPool soundPool;
    private int soundId;

	public CameraPreview(Context context, AttributeSet attributeSet) {
		super(context, attributeSet);
		holder = getHolder();
		holder.addCallback(this);
		holder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
		pictureCallback = new PictureCallback(context);
	}

	public void setPreviewCallback(PreviewCallback previewCallback) {
		this.previewCallback = previewCallback;
		if (camera != null) {
			camera.stopPreview();
			camera.setPreviewCallbackWithBuffer(previewCallback);
			byte[] callbackBuffer = getCallbackBuffer(camera);
			camera.addCallbackBuffer(callbackBuffer);
			camera.startPreview();
		}
	}

	@Override
	public void surfaceChanged(SurfaceHolder surfaceHolder, int format, int width, int height) {
		Camera.Parameters parameters = camera.getParameters();
		Camera.Size previewSize = parameters.getSupportedPreviewSizes().get(0);
		parameters.setPreviewSize(previewSize.width, previewSize.height);
		Camera.Size pictureSize = parameters.getSupportedPictureSizes().get(0);
		parameters.setPictureSize(pictureSize.width, pictureSize.height);
		parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
		camera.setParameters(parameters);
		if (previewCallback != null) {
			camera.setPreviewCallbackWithBuffer(previewCallback);
			byte[] callbackBuffer = getCallbackBuffer(camera);
			camera.addCallbackBuffer(callbackBuffer);
		}
		camera.startPreview();

		camera.autoFocus(new AutoFocusCallback() {
			@Override
			public void onAutoFocus(boolean success, Camera camera) {
			}});
	}

	@Override
	public void surfaceCreated(SurfaceHolder surfaceHolder) {
		if (camera != null) { // to make sure.
			return;
		}

        camera = Camera.open();

        try {
			camera.setPreviewDisplay(surfaceHolder);
		} catch (IOException e) {
			camera.release();
			camera = null;
		}

        soundPool = new SoundPool(1, AudioManager.STREAM_MUSIC, 0);
        soundId = soundPool.load(getContext(), R.raw.picture_taken, 1);
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder surfaceHolder) {
		camera.stopPreview();
		camera.release();
		camera = null;
		soundPool.release();
	}

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            if (camera != null) {
                camera.setPreviewCallbackWithBuffer(null);
                camera.takePicture(null, null, pictureCallback);
                if (previewCallback != null) {
                     camera.setPreviewCallbackWithBuffer(previewCallback);
                     byte[] callbackBuffer = getCallbackBuffer(camera);
                     camera.addCallbackBuffer(callbackBuffer);
                }
                soundPool.play(soundId, 1.0f, 1.0f, 1, 0, 1.0f);
            }
        }
        return true;
    }

    private byte[] getCallbackBuffer(Camera camera) {
		Camera.Parameters param = camera.getParameters();
		int imgformat = param.getPreviewFormat();
		int bitsperpixel = ImageFormat.getBitsPerPixel(imgformat);
		int byteperpixel = bitsperpixel / 8;
		Camera.Size camerasize = param.getPreviewSize();
		int frame_size = camerasize.width * camerasize.height * byteperpixel * 8;

		return new byte[frame_size];
    }
}
