package org.monaka.book.android.camera;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.RectF;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class PreviewCallback extends FaceDetect implements
		android.hardware.Camera.PreviewCallback {

	static {
		System.loadLibrary("yuv2rgb");
	}

	private SurfaceView surfaceView;
	private Bitmap kaodashiBitmap;

	public PreviewCallback(Bitmap kaodashiBitmap, SurfaceView surfaceView) {
		this.surfaceView = surfaceView;
		this.kaodashiBitmap = kaodashiBitmap;
	}

	@Override
	public void onPreviewFrame(byte[] data, Camera camera) {
		if (data != null) {
			Size previewSize = camera.getParameters().getPreviewSize();
			int width = previewSize.width;
			int height = previewSize.height;

			int[] rgb = new int[width * height];
			Bitmap cameraBitmap = Bitmap.createBitmap(width, height,
					Bitmap.Config.RGB_565);
			decodeYUV420SP(rgb, data, width, height);
			cameraBitmap.setPixels(rgb, 0, width, 0, 0, width, height);

			RectF[] rectFArray = faceDetect(cameraBitmap, 1);
			SurfaceHolder surfaceHolder = surfaceView.getHolder();
			Canvas canvas = surfaceHolder.lockCanvas();

			for (int i = 0; i < rectFArray.length; i++) {
				canvas.drawBitmap(kaodashiBitmap, null, rectFArray[i], null);
			}

			surfaceHolder.unlockCanvasAndPost(canvas);
		}

		camera.addCallbackBuffer(data);
	}

	private static final int rangeLimit(int v) {
		if (v < 0) {
			v = 0;
		} else if (v > 262143) {
			v = 262143;
		}
		
		return v;
	}
	
	/**
	 * YUV420 to BMP
	 *
	 * @param rgb
	 * @param yuv420sp
	 * @param width
	 * @param height
	 */
	public static final native void decodeYUV420SP(int[] rgb, byte[] yuv420sp,
			int width, int height);
}
