package org.monaka.book.android.camera;

import wiiremotej.BalanceBoard;
import wiiremotej.MassConstants;
import wiiremotej.WiiRemoteJ;
import wiiremotej.event.BBMassEvent;
import wiiremotej.event.BalanceBoardAdapter;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.Log;
import android.view.SurfaceHolder;

public class BalanceWiiBoardThread extends Thread {
	private static final String TAG = BalanceWiiBoardThread.class.getName();
	boolean active = true;
	private Paint paint = new Paint();
	private SurfaceHolder surfaceHolder;
	private Bitmap bitmap;

	public BalanceWiiBoardThread(SurfaceHolder surfaceHolder, Bitmap bitmap) {
		this.surfaceHolder = surfaceHolder;
		this.bitmap = bitmap;
	}

	public void run() {
		BalanceBoard balanceBoard;

		balanceBoard = findBalanceBoard();
		balanceBoard.addBalanceBoardListener(new BalanceBoardAdapter() {
			@Override
			public void massInputReceived(BBMassEvent evt) {
				double totalMass = evt.getTotalMass();
				double topLeft = evt.getMass(MassConstants.TOP,
						MassConstants.LEFT);
				double topRight = evt.getMass(MassConstants.TOP,
						MassConstants.RIGHT);
				double bottomLeft = evt.getMass(MassConstants.BOTTOM,
						MassConstants.LEFT);
				double bottomRight = evt.getMass(MassConstants.BOTTOM,
						MassConstants.RIGHT);

				double lr = ((topRight + bottomRight) - (topLeft + bottomLeft))
						/ totalMass;
				double tb = ((topRight - topLeft) - (bottomRight + bottomLeft))
						/ totalMass;

				Canvas canvas;
				canvas = surfaceHolder.lockCanvas();
				int x = (int) (canvas.getWidth() * lr);
				int y = (int) (canvas.getHeight() * tb);
				canvas.drawBitmap(bitmap, x, y, paint);
				surfaceHolder.unlockCanvasAndPost(canvas);
			}
		});

		while (active) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		}
	}

	private BalanceBoard findBalanceBoard() {
		BalanceBoard balanceBoard;

		while (active) {
			try {
				balanceBoard = WiiRemoteJ.findBalanceBoard();
				return balanceBoard;
			} catch (Exception e) {
				Log.e(TAG, "Failed to find BalanceBoard.", e);
			}
		}

		return null;
	}
	
	public void cancel() {
		active = false;
	}
}
