package org.monaka.book.android.camera;

import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.net.Uri;
import android.provider.MediaStore;

public class PictureCallback extends FaceDetect implements Camera.PictureCallback {
	private Context context;

	public PictureCallback(Context context) {
		this.context = context;
	}

	@Override
	public void onPictureTaken(byte[] data, Camera camera) {
		if (data != null) {
			BitmapFactory.Options options = new BitmapFactory.Options();
			Bitmap cameraBitmap = BitmapFactory.decodeByteArray(data, 0,
					data.length, options);
			Bitmap kaodashiBitmap = BitmapFactory.decodeResource(
					context.getResources(), R.drawable.aduchan);


			Camera.Parameters parameters = camera.getParameters();
			Size previewSize = parameters.getPreviewSize();

			Bitmap offBitmap = Bitmap.createBitmap(previewSize.width,
					previewSize.height, Bitmap.Config.ARGB_8888);
			Canvas offScreen = new Canvas(offBitmap);
			Rect rect = new Rect(0, 0, previewSize.width, previewSize.height);
			offScreen.drawBitmap(cameraBitmap, null, rect, (Paint) null);

			RectF[] rectFArray = faceDetect(cameraBitmap, 1);

			for (int i = 0; i < rectFArray.length; i++) {
				offScreen.drawBitmap(kaodashiBitmap, null, rectFArray[i], null);
			}

			String contentUriString = MediaStore.Images.Media.insertImage(
					context.getContentResolver(), offBitmap, "", "");

			if (contentUriString != null) {
				Uri uri = getFileUri(contentUriString);
				Intent intent = new Intent(
						Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, uri);

				context.sendBroadcast(intent);
			}
		}
		camera.startPreview();
	}

	Uri getFileUri(String contentUriString) {
		Uri uri;

		uri = Uri.parse(contentUriString);

		ContentResolver contentResolver = context.getContentResolver();
		Cursor c = contentResolver.query(uri, null, null, null, null);

		c.moveToFirst();

		int index = c.getColumnIndex(MediaStore.MediaColumns.DATA);
		String fileUriString = "file://" + c.getString(index);

		uri = Uri.parse(fileUriString);

		return uri;
	}

}
