package org.monaka.book.android.camera;

import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.hardware.usb.UsbAccessory;
import android.hardware.usb.UsbManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.ParcelFileDescriptor;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;

public class CameraActivity extends Activity {
	private static final String ACTION_USB_PERMISSION = "org.monaka.book.android.camera.adk.permission";
	private MediaPlayer mediaPlayer;
	private SensorManager sensorManager;
	private ParcelFileDescriptor mFileDescriptor;
	private UsbAccessory mAccessory;
	private InputStream mInputStream;
	private OutputStream mOutputStream;


	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		registerAdk();

		getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.main);

		CameraPreview cameraPreview = (CameraPreview) findViewById(R.id.CameraPreview);
		SurfaceView surfaceView = (SurfaceView) findViewById(R.id.surfaceView);

		Bitmap kaodashiBitmap = BitmapFactory.decodeResource(getResources(),
				R.drawable.aduchan);
		PreviewCallback previewCallback = new PreviewCallback(kaodashiBitmap,
				surfaceView);
		cameraPreview.setPreviewCallback(previewCallback);
	}

	private void registerAdk() {
		IntentFilter filter = new IntentFilter();
		filter.addAction(ACTION_USB_PERMISSION);
		filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
		registerReceiver(usbReceiver, filter);
	}

	@Override
	protected void onPause() {
		super.onPause();

		mediaPlayer.stop();
		mediaPlayer.release();
	}

	@Override
	protected void onResume() {
		super.onResume();
		Context context = getApplicationContext();
		mediaPlayer = MediaPlayer.create(context, R.raw.kaodashi_bgm);
		mediaPlayer.setLooping(true);
		mediaPlayer.start();

		sensorManager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);
		List<Sensor> sensors = sensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
		if (sensors != null && sensors.size() > 0) {
			Sensor s = sensors.get(0);
			SurfaceHolder surfaceHolder = ((SurfaceView)findViewById(R.id.HanabiraView)).getHolder();
			Bitmap bitmap = BitmapFactory.decodeResource(getResources(), R.drawable.hanabira);
			HanabiraListener hanabiraListener = new HanabiraListener(surfaceHolder, bitmap);
			sensorManager.registerListener(hanabiraListener, s, SensorManager.SENSOR_DELAY_UI);
		}
	}

	private void openAccessory(UsbAccessory accessory) {
		UsbManager usbManager = (UsbManager)getSystemService(Context.USB_SERVICE);
		mFileDescriptor = usbManager.openAccessory(accessory);
		if (mFileDescriptor != null) {
			mAccessory = accessory;
			FileDescriptor fd = mFileDescriptor.getFileDescriptor();
			
			mInputStream = new FileInputStream(fd);
			mOutputStream = new FileOutputStream(fd);
		}
	}

	private void closeAccessory() {
		try {
			if (mFileDescriptor != null) {
				mFileDescriptor.close();
			}
		} catch (IOException e) {
		} finally {
			mFileDescriptor = null;
			mAccessory = null;
		}
	}

	private final BroadcastReceiver usbReceiver = new BroadcastReceiver() {

		private void permission(Context context, Intent intent) {
			synchronized (this) {
				UsbAccessory accessory = (UsbAccessory) intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);
				if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
					openAccessory(accessory);
				} else {
					/* error - but do nothing now. Just the doll made silent. */
				}
			}
		}

		
		private void detached(Context context, Intent intent) {
			UsbAccessory accessory = (UsbAccessory) intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);
			if (accessory != null && accessory.equals(mAccessory)) {
				closeAccessory();
			}
		}
		
		@Override
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			if (ACTION_USB_PERMISSION.equals(action)) {
				permission(context, intent);
			} else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) {
				detached(context, intent);
			}
		}


	};
}
