package org.monaka.book.android.camera;

import java.util.List;
import java.util.Set;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;

public class CameraActivity extends Activity {
	private MediaPlayer mediaPlayer;
	private SensorManager sensorManager;
	private BluetoothAdapter bluetoothAdapter;
	private final int REQUEST_ENABLE_BT = 1;
	private BluetoothThread bluetoothThread;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.main);

		CameraPreview cameraPreview = (CameraPreview) findViewById(R.id.CameraPreview);
		SurfaceView surfaceView = (SurfaceView) findViewById(R.id.surfaceView);

		Bitmap kaodashiBitmap = BitmapFactory.decodeResource(getResources(),
				R.drawable.aduchan);
		PreviewCallback previewCallback = new PreviewCallback(kaodashiBitmap,
				surfaceView);
		cameraPreview.setPreviewCallback(previewCallback);

		bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		if (bluetoothAdapter == null) {

		}

		if (!bluetoothAdapter.isEnabled()) {
			Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(intent, REQUEST_ENABLE_BT);
		}
		setupSppDevice();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == REQUEST_ENABLE_BT) {
			if (resultCode == Activity.RESULT_OK) {
				setupSppDevice();
			} else {
				Log.d("org.monaka.book.android.camera", "Bluetooth not enabled");
				finish();
			}
		}
	}

	private void setupSppDevice() {
		Set<BluetoothDevice> pairedDevicesSet = bluetoothAdapter
				.getBondedDevices();
		if (pairedDevicesSet.size() > 0) {
			BluetoothDevice[] pairedDevices = (BluetoothDevice[]) pairedDevicesSet
					.toArray(new BluetoothDevice[pairedDevicesSet.size()]);
			bluetoothThread = new BluetoothThread(pairedDevices[0]);
			bluetoothThread.start();
		}
	}

	@Override
	protected void onPause() {
		super.onPause();

		mediaPlayer.stop();
		mediaPlayer.release();
	}

	@Override
	protected void onResume() {
		super.onResume();
		Context context = getApplicationContext();
		mediaPlayer = MediaPlayer.create(context, R.raw.kaodashi_bgm);
		mediaPlayer.setLooping(true);
		mediaPlayer.start();

		sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
		List<Sensor> sensors = sensorManager
				.getSensorList(Sensor.TYPE_ACCELEROMETER);
		if (sensors != null && sensors.size() > 0) {
			Sensor s = sensors.get(0);
			SurfaceHolder surfaceHolder = ((SurfaceView) findViewById(R.id.HanabiraView))
					.getHolder();
			Bitmap bitmap = BitmapFactory.decodeResource(getResources(),
					R.drawable.hanabira);
			HanabiraListener hanabiraListener = new HanabiraListener(
					surfaceHolder, bitmap);
			sensorManager.registerListener(hanabiraListener, s,
					SensorManager.SENSOR_DELAY_UI);
		}
	}
}
